
# Launcher for programs and calculator
#
# plugin name: 
#       launcher
#
# available options:
#       :history_size - (number) number of last choices to keep in history (first choices)
#                       default value is 4
#       :show_output - if false (default) all output from launched program will be redirected
#                       to /dev/null
#
# available commands:
#       - no commmand - default mode (launcher + calculator) - if user entry begins with a 
#                       number, calculator mode is activated
#       - launcher    - just program launcher
#       - calculator  - just a calculator
#
#       - update      - update program list
#
# usage:
#   When in default mode, if user entry starts with a number calculator will be enabled. Else,
#   program will be launched.
#
#   Calculator:
#     Launcher will return value of entered expression. Returned value can be used again in 
#     two ways:
#       - start your entry with an operation (for example +4, *6, **2)
#       - use variable ans in an expression (for example 3+sqrt(ans)-2)
#     You can use all math functions provided by Ruby's Math module. Examples are sin, cos,
#     tan, sqrt, sinh, cosh, erfc, ... Power is expressed with ** operator (2**3=8)
#
#     Note: all numbers without decimal point are considered integers. For example, 9/4 will
#     return 2.0 (after the first calculation, result is converted to float). If you write 
#     9/4.0 then it will return 2.25
#
#     Example:
#       [Calculator]  2+2
#       4.0     *3
#       12.0    +2**3
#       20.0    ans*sqrt(9)
#       60.0
#
#  Launcher:
#    For simple program launching, just type a part of program name, select it with arrows if
#    needed, and press return.
#
#    If you need to start program with some arguments type full program name (you can type part
#    of it and then press tab to complete it) and then (after space) type arguments.
#
#    You can also run a program with file (or directory) passed to it. After program name press
#    space and then press return. After that you'll get prompt with file browser (for start you
#    can choose between root directory and your home directory). When you get to the wanted file
#    just press return and program will be launched with it's path as an argument. If you'd like
#    to pass directory as an argument, enter that directory and press escape. This also works 
#    when you add arguments after program name.
#
#    Example:
#       [Run] evince -f <return>
#       [evince -f ]          /  ~                      < type / >
#       [evince -f /]         /bin /boot /dev ...       < type tmp >
#       [evince -f /tmp]      ...
#       ...
#

#
# Copyright 2010 Igor Brkic <igor@hyperglitch.com>
#

include Math

class Launcher

    def initialize io
        @io=io

        @history=[]

        # update app list on initialization
        plist_update
    end

    def setup options
        @history_max_size=options[:history_size]||4
        @show_output=options[:show_output]||false
    end

    def execute app
        if @show_output then
            redirect=''
        else
            redirect=' &> /dev/null '
        end

        system app+redirect+' &'
        @io.log "[Launcher] executed '"+app+"'"
    end

    def calculator resp=''
        if resp=='' then
            resp=@io.input([], '[Calculator]')
        end
        r2=resp
        r=0.0
        while r2!='' do
            if r2.gsub! "ans", r.to_s then
                input=r2
            elsif not r2.scan(/^[a-zA-Z0-9].*/).empty? then
                # doesn't start with operation
                input=r2
            else
                input=r.to_s+r2
            end

            # evaluate expression
            begin
                r=eval(input).to_f
            rescue Exception
                r='ERR'
            end

            r2=@io.input([], r.to_s)
        end
    end

    # update program list
    def plist_update
        # inspired by ruby-wmii :)
        @plist = {}
        path_glob = ENV["PATH"].gsub(/,/, '\\,').tr(":",",")

        Dir.glob("{#{path_glob}}/*").select do |fname|
            File.file?(fname) && File.executable?(fname)
        end.uniq.each{|v| @plist[File.basename(v)]=v}
        @io.log '[Launcher] program list updated'
    end

    def runner resp='', calc=true
        if resp=='' then
            resp=@io.input @history+@plist.keys.sort, "[Run]"

            if !resp.scan(/^[0-9]/).empty? and calc then 
                calculator resp
                return 
            else
                h=resp.sub(/ +$/, '')
                @history.unshift h unless @history.member? h
                @history.pop if @history.length>@history_max_size
            end
        end

        return nil if resp==''

        # if input ends with space, path to file needs to be set too
        if resp[-1].chr==' ' then

            path=''
            list={'~'=>ENV['HOME']+'/', '/'=>'/'}
            show=list.keys

            while true do
                r=@io.input show, '['+resp+' '+path+']'

                if r.empty? or File.file? path+r then
                    #execute @plist[resp.gsub(/ /, '')]+' "'+path+r+'"'
                    tmp=resp.split
                    execute @plist[tmp[0]]+(tmp.length>1 ? ' '+tmp[1..-1].join(' ') : '' )+' "'+path+r+'"'
                    break
                elsif list.member? r
                    ls=case r
                       when '../':
                           path.sub! /[^\/]+\/$/, ''
                           Dir.glob path+'*'
                       else
                           path=list[r]
                           Dir.glob list[r]+'*'
                       end

                    dirs={}; files={}
                    ls.select{|f| File.directory?(f)}.each{|d| dirs[File.basename(d)+'/']=d+'/'}
                    (ls-dirs.values.map{|w| w.chop}).each{|f| files[File.basename(f)]=f}

                    dirs['../']='../'
                    show=dirs.keys.sort+files.keys.sort
                    list=dirs.merge files
                else
                    break
                end
            end

        else
            # just run it (if it is legal app name)
            resp=resp.split
            execute @plist[resp[0]] + (resp.length>1 ? ' '+resp[1..-1].join(' ') : '') if @plist.member? resp[0]
        end
    end

    def run action=[]
        action=action[0]

        case action
        when 'update':
            plist_update

        when 'calculator':
            calculator

        when 'launcher':
            runner '', calc=false
        
        else
            runner
        end
    end
end
